@@ -11,11 +11,13 @@
 
 #include <glib/gi18n.h>
 
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
 #include "gduapplication.h"
+#include "gduformatvolumedialog.h"
 #include "gduwindow.h"
 #include "gdulocaljob.h"
 
@@ -111,6 +113,47 @@ gdu_application_ensure_client (GduApplication *app)
  out:
   ;
 }
+
+static UDisksObject *
+gdu_application_object_from_block_device (GduApplication *app,
+                                          const gchar *block_device,
+                                          gchar **error_message)
+{
+  struct stat statbuf;
+  const gchar *crypto_backing_device;
+  UDisksObject *object, *crypto_backing_object;
+  UDisksBlock *block;
+
+  object = NULL;
+
+  if (stat (block_device, &statbuf) != 0)
+    {
+      *error_message = g_strdup_printf (_("Error opening %s: %s\n"), block_device, g_strerror (errno));
+      goto out;
+    }
+
+  block = udisks_client_get_block_for_dev (app->client, statbuf.st_rdev);
+  if (block == NULL)
+    {
+      *error_message = g_strdup_printf (_("Error looking up block device for %s\n"), block_device);
+      goto out;
+    }
+
+  object = UDISKS_OBJECT (g_dbus_interface_dup_object (G_DBUS_INTERFACE (block)));
+  g_object_unref (block);
+
+  crypto_backing_device = udisks_block_get_crypto_backing_device ((udisks_object_peek_block (object)));
+  crypto_backing_object = udisks_client_get_object (app->client, crypto_backing_device);
+  if (crypto_backing_object != NULL)
+    {
+      g_object_unref (object);
+      object = crypto_backing_object;
+    }
+
+ out:
+  return object;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 /* called in primary instance */
@@ -126,11 +169,15 @@ gdu_application_command_line (GApplication            *_app,
   gint argc;
   gint ret = 1;
   gchar *s;
-  gchar *opt_block_device = NULL;
+  gchar *opt_block_device = NULL, *error_message = NULL;
   gboolean opt_help = FALSE;
+  gboolean opt_format = FALSE;
+  gint opt_xid = -1;
   GOptionEntry opt_entries[] =
   {
     {"block-device", 0, 0, G_OPTION_ARG_STRING, &opt_block_device, N_("Select device"), NULL },
+    {"format-device", 0, 0, G_OPTION_ARG_NONE, &opt_format, N_("Format selected device"), NULL },
+    {"xid", 0, 0, G_OPTION_ARG_INT, &opt_xid, N_("Parent window XID for the format dialog"), NULL },
     {"help", '?', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &opt_help, N_("Show help options"), NULL },
     {NULL}
   };
@@ -158,43 +205,54 @@ gdu_application_command_line (GApplication            *_app,
       goto out;
     }
 
+  if (opt_format && opt_block_device == NULL)
+    {
+      g_application_command_line_printerr (command_line, _("--format-device must be used together with --block-device\n"));
+      goto out;
+    }
+
+  if (opt_xid != -1 && !opt_format)
+    {
+      g_application_command_line_printerr (command_line, _("--format-device must be specified when using --xid\n"));
+      goto out;
+    }
+
+  gdu_application_ensure_client (app);
+
   if (opt_block_device != NULL)
     {
-      struct stat statbuf;
-      UDisksBlock *block;
-
-      if (stat (opt_block_device, &statbuf) != 0)
+      object_to_select = gdu_application_object_from_block_device (app, opt_block_device, &error_message);
+      if (object_to_select == NULL)
         {
-          g_application_command_line_print (command_line, _("Error opening %s: %m\n"), opt_block_device);
+          g_application_command_line_print (command_line, "%s", error_message);
+          g_free (error_message);
           goto out;
         }
+    }
 
-      gdu_application_ensure_client (app);
-
-      block = udisks_client_get_block_for_dev (app->client, statbuf.st_rdev);
-      if (block == NULL)
+  if (opt_xid == -1)
+    {
+      if (app->window == NULL)
         {
-          g_application_command_line_print (command_line, _("Error looking up block device for %s: %m\n"),
-                                            opt_block_device);
-          goto out;
+          g_application_activate (G_APPLICATION (app));
+        }
+      else
+        {
+          /* TODO: startup notification stuff */
+          gtk_window_present (GTK_WINDOW (app->window));
         }
 
-      object_to_select = UDISKS_OBJECT (g_dbus_interface_dup_object (G_DBUS_INTERFACE (block)));
-      g_object_unref (block);
+      if (object_to_select != NULL)
+        {
+          gdu_window_select_object (app->window, object_to_select);
+          if (opt_format)
+            gdu_format_volume_dialog_show (app->window, object_to_select);
+        }
     }
-
-  if (app->window == NULL)
+  else if (opt_format)
     {
-      g_application_activate (G_APPLICATION (app));
+      gdu_format_volume_dialog_show_for_xid (app->client, opt_xid, object_to_select);
     }
-  else
-    {
-      /* TODO: startup notification stuff */
-      gtk_window_present (GTK_WINDOW (app->window));
-    }
-
-  if (object_to_select != NULL)
-    gdu_window_select_object (app->window, object_to_select);
 
 
   ret = 0;
