@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*-
  *
- * Copyright (C) 2008-2012 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * Licensed under GPL version 2 or later.
  *
@@ -77,6 +77,7 @@ typedef struct
   guint64 num_error_bytes;
   gint64 start_time_usec;
   gint64 end_time_usec;
+  gboolean played_read_error_sound;
 
   guint update_id;
   GError *copy_error;
@@ -290,6 +291,24 @@ create_disk_image_populate (DialogData *data)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+play_read_error_sound (DialogData *data)
+{
+  const gchar *sound_message;
+
+  /* Translators: A descriptive string for the sound played when
+   * there's a read error that's being ignored, see
+   * CA_PROP_EVENT_DESCRIPTION
+   */
+  sound_message = _("Disk image read error");
+  ca_gtk_play_for_widget (GTK_WIDGET (data->window), 0,
+                          CA_PROP_EVENT_ID, "dialog-warning",
+                          CA_PROP_EVENT_DESCRIPTION, sound_message,
+                          NULL);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void
 update_job (DialogData *data,
             gboolean    done)
@@ -366,6 +385,13 @@ update_job (DialogData *data,
       gdu_local_job_set_extra_markup (data->local_job, extra_markup);
     }
 
+  /* Play a sound the first time we encounter a read error */
+  if (num_error_bytes > 0 && !data->played_read_error_sound)
+    {
+      play_read_error_sound (data);
+      data->played_read_error_sound = TRUE;
+    }
+
   g_free (extra_markup);
 }
 
@@ -429,6 +455,59 @@ on_success (gpointer user_data)
   dialog_data_uninhibit (data);
   dialog_data_complete_and_unref (data);
 
+  /* OK, we're done but we had to replace unreadable data with
+   * zeroes. Bring up a modal dialog to inform the user of this and
+   * allow him to delete the file, if so desired.
+   */
+  if (data->num_error_bytes > 0)
+    {
+      GtkWidget *dialog, *button;
+      GError *error = NULL;
+      gchar *s = NULL;
+      gint response;
+      gdouble percentage;
+
+      dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (data->window),
+                                                   GTK_DIALOG_MODAL,
+                                                   GTK_MESSAGE_WARNING,
+                                                   GTK_BUTTONS_CLOSE,
+                                                   "<big><b>%s</b></big>",
+                                                   /* Translators: Primary message in dialog shown if some data was unreadable while creating a disk image */
+                                                   _("Unrecoverable read errors while creating disk image"));
+      s = g_format_size (data->num_error_bytes);
+      percentage = 100.0 * ((gdouble) data->num_error_bytes) / ((gdouble) gdu_estimator_get_completed_bytes (data->estimator));
+      gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
+                                                  /* Translators: Secondary message in dialog shown if some data was unreadable while creating a disk image.
+                                                   * The %f is the percentage of unreadable data (ex. 13.0).
+                                                   * The first %s is the amount of unreadable data (ex. "4.2 MB").
+                                                   * The second %s is the name of the device (ex "/dev/").
+                                                   */
+                                                  _("%2.1f%% (%s) of the data on the device “%s” was unreadable and replaced with zeroes in the created disk image file. This typically happens if the medium is scratched or if there is physical damage to the drive"),
+                                                  percentage,
+                                                  s,
+                                                  gtk_label_get_text (GTK_LABEL (data->source_label)));
+      button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+                                      /* Translators: Label of secondary button in dialog if some data was unreadable while creating a disk image */
+                                      _("_Delete Disk Image File"),
+                                      GTK_RESPONSE_NO);
+      gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (gtk_dialog_get_action_area (GTK_DIALOG (dialog))),
+                                          button, TRUE);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+      response = gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      g_free (s);
+
+      if (response == GTK_RESPONSE_NO)
+        {
+          if (!g_file_delete (data->output_file, NULL, &error))
+            {
+              g_warning ("Error deleting file: %s (%s, %d)",
+                         error->message, g_quark_to_string (error->domain), error->code);
+              g_clear_error (&error);
+            }
+        }
+    }
+
   dialog_data_unref (data);
   return FALSE; /* remove source */
 }
