@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*-
  *
- * Copyright (C) 2008-2012 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * Licensed under GPL version 2 or later.
  *
@@ -1146,6 +1146,10 @@ update_mdraid (GduDeviceTreeModel *model,
 
   jobs_running = mdraid_has_jobs (model, mdraid);
 
+  /* also show the spinner if a sync op is in progress */
+  if (udisks_mdraid_get_sync_completed (mdraid) > 0.0)
+    jobs_running = TRUE;
+
   gtk_tree_model_get (GTK_TREE_MODEL (model),
                       &iter,
                       GDU_DEVICE_TREE_MODEL_COLUMN_PULSE, &pulse,
